"Dialog and data used for the generic search dialog."


#
#   MoviePaca, movie database manager.
#   Copyright (C) 2010  Joakim Soderlund
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


import gtk
import os.path

import util


# Operators for the search.
OPERATOR = ["<", "<=", "=", ">=", ">", "LIKE", "IS"]
TYPE = {
		"Movie year" : {
			"table" : "Movie",
			"attribute" : "year",
		},
		"Movie title" :  {
			"table" : "Movie",
			"attribute" : "title",
		},
		"Genre name" : {
			"table" : "Genre",
			"attribute" : "genre",
		},
		"Person name" : {
			"table" : "Person",
			"attribute" : "name",
		},
		"Role name" : {
			"table" : "Role",
			"attribute" : "role",
		},
		"Movie duration" : {
			"table" : "Movie",
			"attribute" : "duration",
		},
		"Movie ID" : {
			"table" : "Movie",
			"attribute" : "movie_id",
		},
		"Person ID" : {
			"table" : "Person",
			"attribute" : "person_id",
		},
		"Genre ID" : {
			"table" : "Genre",
			"attribute" : "genre_id",
		},
		"Role ID" : {
			"table" : "Role",
			"attribute" : "role_id",
		},
}

# Data for movie searches.
MOVIE = {
	"from" : "Movie",
	"select" : ["movie_id", "title", "year", "duration"],
	"operator" : OPERATOR,
	"type" : TYPE,
	"group" : "movie_id",
	"join" : {
		"MovieStaff" : [],
		"Person" : ["MovieStaff"],
		"Role" : ["MovieStaff"],
		"MovieGenre" : [],
		"Genre" : ["MovieGenre"],
	},
}

# Data for person searches.
PERSON = {
	"from" : "Person",
	"select" : ["person_id", "name"],
	"operator" : OPERATOR,
	"type" : TYPE,
	"group" : "person_id",
	"join" : {
		"MovieStaff" : [],
		"Movie" : ["MovieStaff"],
		"Role" : ["MovieStaff"],
		"MovieGenre" : [],
		"Genre" : ["MovieGenre"],
	},
}

GENRE = {
	"from" : "Genre",
	"select" : ["genre_id", "genre"],
	"operator" : OPERATOR,
	"type" : TYPE,
	"group" : "genre_id",
	"join" : {
		"MovieStaff" : ["MovieGenre"],
		"Movie" : ["MovieGenre"],
		"Role" : ["MovieStaff"],
		"MovieGenre" : [],
		"Person" : ["MovieStaff"],
	},
}

ROLE = {
	"from" : "Role",
	"select" : ["role_id", "role"],
	"operator" : OPERATOR,
	"type" : TYPE,
	"group" : "role_id",
	"join" : {
		"MovieStaff" : [],
		"Movie" : ["MovieStaff"],
		"Genre" : ["MovieGenre"],
		"MovieGenre" : ["MovieStaff"],
		"Person" : ["MovieStaff"],
	},
}


class DialogSearch():
	def __init__(self, searches, callback):
		self.searches = searches
		self.callback = callback
		self.builder = gtk.Builder()
		self.builder.add_from_file(os.path.join(util.DATA_PATH, "glade", "search.glade"))
		self.builder.connect_signals(self)
		self.dialog = self.builder.get_object("dia_search")
		self.tw_conditions = self.builder.get_object("tw_conditions")
		self.ls_conditions = self.builder.get_object("ls_conditions")
		
		self.tw_conditions.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
		
		self.renderers = [
			self.builder.get_object("renderer_type"),
			self.builder.get_object("renderer_operator"),
			self.builder.get_object("renderer_data"),
		]
		
		ls_type = self.builder.get_object("ls_type")
		for conType in self.searches["type"].keys():
			ls_type.append([conType])
		ls_type.set_sort_column_id(0, gtk.SORT_ASCENDING)
		
		ls_operator = self.builder.get_object("ls_operator")
		for conOper in self.searches["operator"]:
			ls_operator.append([conOper])
		
	def search_cancel(self, *args):
		self.dialog.destroy()
	
	def search_perform(self, *args):
		query_data = []
		query_string = [
			"SELECT", 
			", ".join(self.searches["select"]),
			"FROM",
			self.searches["from"]
		]
		
		# Add joins
		tables = set()
		for row in self.ls_conditions:
			rowType = row[0]
			if rowType not in self.searches["type"]:
				dialog = gtk.MessageDialog(
					self.dialog,
					gtk.DIALOG_MODAL,
					gtk.MESSAGE_ERROR,
					gtk.BUTTONS_OK,
					"Invalid type.",
				)
				dialog.run()
				dialog.destroy()
				return
			
			tables.add(self.searches["type"][rowType]["table"])
		
		joins = list()
		for table in tables:
			if table != self.searches["from"] and table not in joins:
				self.resolve_join(joins, table)
		
		for join in joins:
			query_string.append("NATURAL JOIN " + join)
		
		# Add conditions.
		conditions = []
		for row in self.ls_conditions:
			row_info = self.searches["type"][row[0]]
			operator = row[1]
			
			if operator not in self.searches["operator"]:
				dialog = gtk.MessageDialog(
					self.dialog,
					gtk.DIALOG_MODAL,
					gtk.MESSAGE_ERROR,
					gtk.BUTTONS_OK,
					"Invalid operator.",
				)
				dialog.run()
				dialog.destroy()
				return
							
			conditions.append("%s %s %%s" % (row_info["attribute"], operator))
			query_data.append(str(row[2]))
		
		if conditions:
			query_string.append("WHERE")
			query_string.append(" AND ".join(conditions))
			
		# Group
		if "group" in self.searches:
			query_string.append("GROUP BY " + self.searches["group"])
			
		# Order
		if "order" in self.searches:
			query_string.append("ORDER BY " + self.searches["order"])

		# Build query.
		self.callback(" ".join(query_string), query_data)
		self.dialog.destroy()

	def resolve_join(self, joins, join):
		for dependency in self.searches["join"][join]:
			if not dependency in joins:
				self.resolve_join(joins, dependency)
		
		joins.append(join)

	def condition_add(self, *args):
		self.ls_conditions.append(("<TYPE>", "LIKE", "<DATA>"))
	
	def condition_remove(self, *args):
		model, rows = self.tw_conditions.get_selection().get_selected_rows()
		iterators = [model.get_iter(row) for row in rows]
		for iterator in iterators:
			model.remove(iterator)

	def cell_edited(self, renderer, path, text):
		if text:
			index = self.renderers.index(renderer)
			iterator = self.ls_conditions.get_iter(path)
			self.ls_conditions.set_value(iterator, index, text)
		
		
